//////////////////////////////////////////////////////////////////////////////
//
// Minimal
// Copyright 2004 by Thierry Tremblay
//
// Minimal Sample for PocketHAL
// For more info: http://www.droneship.com
//
//////////////////////////////////////////////////////////////////////////////

#include "GameCore.h"
#include "utility.h"



//////////////////////////////////////////////////////////////////////////////
//
// Declare the program's entry point
//
//////////////////////////////////////////////////////////////////////////////

PHAL_DECLARE_ENTRY_POINT( GameCore );


BOOL g_bDoPhysics = TRUE;

DWORD WINAPI PhysicsThread(LPVOID p_pGameCore)
{
	GameCore*	l_pGameCore = ((GameCore*)p_pGameCore);
	while (g_bDoPhysics)
	{
		l_pGameCore->DoPhysics();
		//Sleep(0); // This Sleep and the one in the OnGameLoop seems to help, why? How? When?...
	}

	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//
// GameCore
//
//////////////////////////////////////////////////////////////////////////////

GameCore::GameCore()
{
    m_config.m_appName     = TEXT("Sample PHAL threading");
    m_config.m_orientation = ORIENTATION_NORMAL;
}


GameCore::~GameCore()
{
	g_bDoPhysics = FALSE;
	DWORD l_dwWaitState = WAIT_TIMEOUT;
	while (  l_dwWaitState != WAIT_OBJECT_0)
	{
		l_dwWaitState = WaitForSingleObject(m_hPhysicsThread, 500);
		if (l_dwWaitState == WAIT_FAILED)
			break;
	}
}

bool GameCore::OnInitialize()
{
    if (!Game::OnInitialize())
        return false;

	// Initializes the left side position of the square
	m_dX = 0;

	// Initialize speed (pixel per milliseconds), and initial tick
	m_dXSpeed = ((double)SQUARE_SPEED) / 1000.;
	m_dwLastSystemTick = GetTickCount();

	// Launches the physic thread
	DWORD l_hThreadID;
	m_hPhysicsThread = CreateThread(NULL, 0, PhysicsThread, (LPVOID)this, 0, &l_hThreadID);

    return true;
}



bool GameCore::OnGameLoop()
{
    Display *display = GetDisplay();

    if (display->BeginScene())
    {
        // Render one frame
        Clear( display->GetBackBuffer(), 0 );
		FillRect(display->GetBackBuffer(), (INT)m_dX, 50, 10, 10, 0xFF00);
        DrawText( display->GetBackBuffer(), 0, 0, GetFPSString() );
        display->Swap();
		//Sleep(0); // Cf. PhysicsThread note.
	}

    return Game::OnGameLoop();
}

VOID GameCore::DoPhysics()
///////////////////////////////////////////////////////////////////
/// \brief
///		Moves the square
/// \param None
///		
/// \return VOID (None)
///
////////////////////////////////////////////////////////////////////
{
	// Process tick count since the last call
	DWORD	l_dwCurrentSystemTick = GetTickCount();
	INT		l_iTickDelta = l_dwCurrentSystemTick - m_dwLastSystemTick;

	// Process new position if needed (and update "last tick count")
	if (l_iTickDelta)
	{
		m_dX += m_dXSpeed * l_iTickDelta;
		m_dwLastSystemTick = l_dwCurrentSystemTick;
	}

	// Check left and right position boundaries
	if (m_dX > MOVING_AREA_WIDTH - SQUARE_SIZE)
	{
		m_dX = MOVING_AREA_WIDTH - SQUARE_SIZE;
		m_dXSpeed = -m_dXSpeed;
	}

	if (m_dX < 0)
	{
		m_dX = 0;
		m_dXSpeed = -m_dXSpeed;
	}
}

VOID FillRect(PHAL::Surface * p_pSurface, INT p_iLeft, INT p_iTop, INT p_iWidth, INT p_iHeight, unsigned __int16 p_iColor)
///////////////////////////////////////////////////////////////////
/// \brief
///		A rectangle filling routine for testing
///
/// \param p_pSurface : destination PHAL surface
/// \param p_iLeft, p_iTop, : destination rectangle position
/// \param p_iWidth, p_iHeight : destination rectangle size
/// \param p_iColor : destination rectangle color
///		
/// \return VOID (None)
///
////////////////////////////////////////////////////////////////////
{
    // Clipping
	RECT m_oSurfaceArea;
    RECT l_oTargetArea;

	SetRect(&m_oSurfaceArea, 0, 0, p_pSurface->GetWidth(), p_pSurface->GetHeight());
	SetRect(&l_oTargetArea, p_iLeft, p_iTop, p_iLeft + p_iWidth, p_iTop + p_iHeight);

	IntersectRect(&l_oTargetArea, &l_oTargetArea, &m_oSurfaceArea);
    if (IsRectEmpty(&l_oTargetArea))
        return;
   
 	// Drawing
	unsigned __int16	*l_pTarget = p_pSurface->GetPixels(l_oTargetArea.left, l_oTargetArea.top);
	INT		x, y, l_iTargetDeltaPitch = (p_pSurface->GetPitch() >> 1) - (l_oTargetArea.right - l_oTargetArea.left);

	for (y = l_oTargetArea.bottom - l_oTargetArea.top; y ; y--)
	{
		for (x = l_oTargetArea.right - l_oTargetArea.left; x; x--) 
		{
			*l_pTarget = p_iColor;
			l_pTarget++;
		}
		l_pTarget += l_iTargetDeltaPitch;
	}
}
